@@ -1,5 +1,5 @@
 /* darkstat 3
- * copyright (c) 2001-2006 Emil Mikulic.
+ * copyright (c) 2001-2011 Emil Mikulic.
  *
  * dns.h: synchronous DNS in a child process.
  *
@@ -7,11 +7,11 @@
  * GNU General Public License version 2. (see COPYING.GPL)
  */
 
-#include <netinet/in.h> /* in_addr_t */
+struct addr;
 
-void dns_init(void);
+void dns_init(const char *privdrop_user);
 void dns_stop(void);
-void dns_queue(const in_addr_t ip);
+void dns_queue(const struct addr *const ipaddr);
 void dns_poll(void);
 
 /* vim:set ts=3 sw=3 tw=78 expandtab: */
