@@ -1,32 +1,25 @@
 /* darkstat 3
- * copyright (c) 2001-2011 Emil Mikulic.
+ * copyright (c) 2001-2012 Emil Mikulic.
  *
  * opt.h: global options
  */
 
-/*
- * Capture options.
- */
+/* Capture options. */
 extern int opt_want_pppoe;
 extern int opt_want_macs;
 extern int opt_want_hexdump;
 extern int opt_want_snaplen;
 extern int opt_wait_secs;
 
-/*
- * Error/logging options.
- */
+/* Error/logging options. */
 extern int opt_want_verbose;
 extern int opt_want_syslog;
 
-/*
- * Accounting options.
- */
+/* Accounting options. */
 extern unsigned int opt_highest_port;
 extern int opt_want_local_only;
 
-/*
- * Hosts table reduction - when the number of entries is about to exceed
+/* Hosts table reduction - when the number of entries is about to exceed
  * <max>, we reduce the table to the top <keep> entries.
  */
 extern unsigned int opt_hosts_max;
@@ -34,10 +27,10 @@ extern unsigned int opt_hosts_keep;
 extern unsigned int opt_ports_max;
 extern unsigned int opt_ports_keep;
 
-/*
- * Hosts output options.
- */
+/* Hosts output options. */
 extern int opt_want_lastseen;
-extern const char *opt_interface;
+
+/* Initialized in cap.c, added to <title> */
+extern char *title_interfaces;
 
 /* vim:set ts=3 sw=3 tw=78 expandtab: */
