@@ -156,8 +156,8 @@ function poll() {
   document.getElementById("graph_reload").innerHTML = "reload graphs";
   killChildren(graphs.msg);
   head = xh.responseXML.childNodes[0];
-  document.getElementById("tb").innerHTML = thousands(head.getAttribute("tb"));
-  document.getElementById("tp").innerHTML = thousands(head.getAttribute("tp"));
+  for (var n in {"tb":0, "tp":0, "pc":0, "pd":0})
+   document.getElementById(n).innerHTML = thousands(head.getAttribute(n));
   document.getElementById("rf").innerHTML = head.getAttribute("rf");
  }
 }
